# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import json
from unittest import TestCase

from mock import patch

from hpecp import APIException, APIItemNotFoundException
from hpecp.k8s_cluster import (
    K8sCluster,
    K8sClusterHostConfig,
    K8sClusterStatus,
)

from .base import BaseTestCase, MockResponse, get_client
from .k8s_cluster_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestClusterList(BaseTestCase):
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_k8sclusters(self, mock_get, mock_post):

        # Makes GET Request: https://127.0.0.1:8080/api/v2/k8sclusters/
        clusters = get_client().k8s_cluster.list()

        # Test that json response is saved in each WorkerK8s object
        # assert client.k8s_cluster.get()[0].json is not None

        # Test WorkerK8sList subscriptable access and property setters
        self.assertEqual(clusters[0].id, "/api/v2/k8scluster/20")
        self.assertEqual(clusters[0].name, "def")
        self.assertEqual(clusters[0].description, "my cluster")
        self.assertEqual(clusters[0].k8s_version, "1.17.0")
        self.assertEqual(clusters[0].created_by_user_id, "/api/v1/user/5")
        self.assertEqual(clusters[0].created_by_user_name, "admin")
        self.assertEqual(clusters[0].created_time, 1588260014)

        self.maxDiff = None
        self.assertListEqual(
            clusters[0].k8shosts_config,
            [
                {"node": "/api/v2/worker/k8shost/4", "role": "worker"},
                {"node": "/api/v2/worker/k8shost/5", "role": "master"},
            ],
        )
        self.assertEqual(clusters[0].admin_kube_config, "xyz==")
        self.assertEqual(clusters[0].dashboard_token, "abc==")
        self.assertEqual(clusters[0].api_endpoint_access, "api:1234")
        self.assertEqual(
            clusters[0].dashboard_endpoint_access, "dashboard:1234"
        )
        # The cluster wasn't created with certs
        # TODO: need a test with this value set
        self.assertIsNone(clusters[0].cert_data)
        self.assertEqual(clusters[0].status, "ready")
        self.assertEqual(clusters[0].status_message, "really ready")
        self.assertDictEqual(
            clusters[0]._links, {"self": {"href": "/api/v2/k8scluster/20"}}
        )

        # Test iterators
        assert [cluster.id for cluster in get_client().k8s_cluster.list()] == [
            "/api/v2/k8scluster/20"
        ]

        with self.assertRaisesRegexp(
            AssertionError, "'columns' parameter must be list"
        ):
            get_client().k8s_cluster.list().tabulate(columns="garbage")

    # pylint: disable=no-method-argument
    def mocked_requests_get_missing_cluster_props(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v2/k8scluster":
            return MockResponse(
                json_data={
                    "_links": {"self": {"href": "/api/v2/k8scluster"}},
                    "_embedded": {
                        "k8sclusters": [
                            {
                                "_links": {
                                    "self": {"href": "/api/v2/k8scluster/20"}
                                },
                                "label": {
                                    "name": "def",
                                    "description": "my cluster",
                                },
                                "k8s_version": "1.17.0",
                                "pod_network_range": "10.192.0.0/12",
                                "service_network_range": "10.96.0.0/12",
                                "pod_dns_domain": "cluster.local",
                                "created_by_user_id": "/api/v1/user/5",
                                "created_by_user_name": "admin",
                                "created_time": 1588260014,
                                "k8shosts_config": [
                                    {
                                        "node": "/api/v2/worker/k8shost/4",
                                        "role": "worker",
                                    },
                                    {
                                        "node": "/api/v2/worker/k8shost/5",
                                        "role": "master",
                                    },
                                ],
                                "status": "ready",
                                "persistent_storage": {"nimble_csi": False},
                            }
                        ]
                    },
                },
                status_code=200,
                headers={},
            )
        raise RuntimeError("Unhandle GET request: " + args[0])

    @patch(
        "requests.get", side_effect=mocked_requests_get_missing_cluster_props
    )
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_k8sclusters_missing_props(self, mock_get, mock_post):

        # Makes GET Request: https://127.0.0.1:8080/api/v2/k8sclusters/
        clusters = get_client().k8s_cluster.list()

        # Test that json response is saved in each WorkerK8s object
        # assert client.k8s_cluster.get()[0].json is not None

        # Test WorkerK8sList subscriptable access and property setters
        self.assertEqual(clusters[0].id, "/api/v2/k8scluster/20")

        self.assertEqual(clusters[0].admin_kube_config, "")
        self.assertEqual(clusters[0].dashboard_token, "")
        self.assertEqual(clusters[0].api_endpoint_access, "")
        self.assertEqual(clusters[0].dashboard_endpoint_access, "")
        self.assertEqual(clusters[0].status_message, "")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_k8sclusters_tabulate_all_columns(self, mock_get, mock_post):

        expected_tabulate_output = (
            "+-----------------------+------+-------------+-------------+--------+--------------------+----------------------+--------------+------------------------------------------------------------------------------------------------------------------+-------------------+-----------------+---------------------+---------------------------+-----------+--------+----------------+---------------------------------------------+\n"  # noqa: E501
            "|          id           | name | description | k8s_version | addons | created_by_user_id | created_by_user_name | created_time |                                                 k8shosts_config                                                  | admin_kube_config | dashboard_token | api_endpoint_access | dashboard_endpoint_access | cert_data | status | status_message |                   _links                    |\n"  # noqa: E501
            "+-----------------------+------+-------------+-------------+--------+--------------------+----------------------+--------------+------------------------------------------------------------------------------------------------------------------+-------------------+-----------------+---------------------+---------------------------+-----------+--------+----------------+---------------------------------------------+\n"  # noqa: E501
            '| /api/v2/k8scluster/20 | def  | my cluster  |   1.17.0    |        |   /api/v1/user/5   |        admin         |  1588260014  | [{"node": "/api/v2/worker/k8shost/4", "role": "worker"}, {"node": "/api/v2/worker/k8shost/5", "role": "master"}] |       xyz==       |      abc==      |      api:1234       |      dashboard:1234       |           | ready  |  really ready  | {"self": {"href": "/api/v2/k8scluster/20"}} |\n'  # noqa: E501
            "+-----------------------+------+-------------+-------------+--------+--------------------+----------------------+--------------+------------------------------------------------------------------------------------------------------------------+-------------------+-----------------+---------------------+---------------------------+-----------+--------+----------------+---------------------------------------------+"  # noqa: E501
        )

        # Patch k8shosts_config method to return the node attribute before the role attribute
        def new_k8shosts_config(self):
            return json.dumps(self.json["k8shosts_config"], sort_keys=True)

        setattr(K8sCluster, "k8shosts_config", property(new_k8shosts_config))

        k8scluster_list = get_client().k8s_cluster.list()

        self.maxDiff = None
        self.assertEqual(
            k8scluster_list.tabulate().replace("'", '"'),
            expected_tabulate_output,
        )  # noqa: E501

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_k8sclusters_tabulate_with_column_list(self, mock_get, mock_post):

        k8scluster_list = get_client().k8s_cluster.list()
        self.assertEqual(
            k8scluster_list.tabulate(["description", "id"]),
            "+-------------+-----------------------+\n"
            "| description |          id           |\n"
            "+-------------+-----------------------+\n"
            "| my cluster  | /api/v2/k8scluster/20 |\n"
            "+-------------+-----------------------+",
        )


class TestCreateCluster(TestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_create(self, mock_post):

        with self.assertRaisesRegexp(
            AssertionError, "'name' must be provided and must be a string"
        ):
            get_client().k8s_cluster.create()

        with self.assertRaisesRegexp(
            AssertionError, "'description' if provided, must be a string"
        ):
            get_client().k8s_cluster.create(name="a", description=1)

        with self.assertRaisesRegexp(
            AssertionError, "'k8s_version' if provided, must be a string"
        ):
            get_client().k8s_cluster.create(name="a", k8s_version=1)

        with self.assertRaisesRegexp(
            AssertionError, "'pod_network_range' must be a string"
        ):
            get_client().k8s_cluster.create(name="a", pod_network_range=1)

        with self.assertRaisesRegexp(
            AssertionError, "'service_network_range' must be a string"
        ):
            get_client().k8s_cluster.create(name="a", service_network_range=1)

        with self.assertRaisesRegexp(
            AssertionError, "'pod_dns_domain' must be a string"
        ):
            get_client().k8s_cluster.create(name="a", pod_dns_domain=1)

        with self.assertRaisesRegexp(
            AssertionError, "'persistent_storage_local' must be True or False"
        ):
            get_client().k8s_cluster.create(
                name="a", persistent_storage_local=1
            )

        with self.assertRaisesRegexp(
            AssertionError,
            "'persistent_storage_nimble_csi' must be True or False",
        ):
            get_client().k8s_cluster.create(
                name="a", persistent_storage_nimble_csi=1
            )

        with self.assertRaisesRegexp(
            AssertionError, "'k8shosts_config' must be a list"
        ):
            get_client().k8s_cluster.create(name="a", k8shosts_config=1)

        with self.assertRaisesRegexp(
            AssertionError, "'k8shosts_config' must have at least one item"
        ):
            get_client().k8s_cluster.create(name="a", k8shosts_config=[])

        with self.assertRaisesRegexp(
            AssertionError,
            "'k8shosts_config' item '0' is not of type K8sClusterHostConfig",
        ):
            get_client().k8s_cluster.create(name="a", k8shosts_config=[1, 2])

        # pylint: disable=anomalous-backslash-in-string
        with self.assertRaisesRegexp(
            AssertionError,
            (
                "'node' must have format"
                " '\/api\/v2\/worker\/k8shost\/\[0-9\]\+'"  # noqa: W605
            ),
        ):
            get_client().k8s_cluster.create(
                name="a", k8shosts_config=[K8sClusterHostConfig("a", "b")]
            )

        with self.assertRaisesRegexp(
            AssertionError,
            "'role' must one of \['master, worker'\]",  # noqa: W605
        ):
            get_client().k8s_cluster.create(
                name="a",
                k8shosts_config=[
                    K8sClusterHostConfig("/api/v2/worker/k8shost/1", "b")
                ],
            )

        # Finally we can create a cluster
        id = get_client().k8s_cluster.create(
            name="a",
            k8shosts_config=[
                K8sClusterHostConfig("/api/v2/worker/k8shost/1", "master")
            ],
        )
        self.assertEqual(id, "/api/v2/k8scluster/99")

        # now with a description
        id = get_client().k8s_cluster.create(
            name="a",
            k8shosts_config=[
                K8sClusterHostConfig("/api/v2/worker/k8shost/1", "master")
            ],
            description="Cluster Description",
        )
        self.assertEqual(id, "/api/v2/k8scluster/99")

        # now with a k8s version
        id = get_client().k8s_cluster.create(
            name="a",
            k8shosts_config=[
                K8sClusterHostConfig("/api/v2/worker/k8shost/1", "master")
            ],
            k8s_version="1.18.0",
        )
        self.assertEqual(id, "/api/v2/k8scluster/99")

        # now with a k8s version
        id = get_client().k8s_cluster.create(
            name="a",
            k8shosts_config=[
                K8sClusterHostConfig("/api/v2/worker/k8shost/1", "master")
            ],
            k8s_version="1.18.0",
            addons=["picasso"],
        )
        self.assertEqual(id, "/api/v2/k8scluster/99")

    def mocked_requests_create_error_post(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/login":
            return MockResponse(
                json_data={},
                status_code=200,
                headers={
                    "location": (
                        "/api/v1/session/df1bfacb-xxxx-xxxx-xxxx-c8f57d8f3c71"
                    )
                },
            )
        elif args[0] == "https://127.0.0.1:8080/api/v2/k8scluster":
            return MockResponse(
                json_data={},
                raise_for_status_flag=True,
                status_code=500,
                headers={},
            )
        raise RuntimeError("Unhandle POST request: " + args[0])

    @patch("requests.post", side_effect=mocked_requests_create_error_post)
    def test_create_with_APIException(self, mock_post):

        with self.assertRaises(APIException):
            get_client().k8s_cluster.create(
                name="a",
                k8shosts_config=[
                    K8sClusterHostConfig("/api/v2/worker/k8shost/1", "master")
                ],
            )


class TestGetCluster(TestCase):
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_k8scluster(self, mock_get, mock_post):

        with self.assertRaises(APIItemNotFoundException):
            get_client().k8s_cluster.get(
                id="/api/v2/k8scluster/999", setup_log=False
            )

        get_client().k8s_cluster.get(
            id="/api/v2/k8scluster/123", setup_log=False
        )

        get_client().k8s_cluster.get(
            id="/api/v2/k8scluster/123", setup_log=True
        )


class TestWaitForClusterStatus(BaseTestCase):

    # pylint: disable=no-method-argument
    # def mocked_requests_get(*args, **kwargs):
    #     if args[0] == "https://127.0.0.1:8080/api/v2/k8scluster/123":
    #         return MockResponse(
    #             json_data={
    #                 "_links": {"self": {"href": "/api/v2/k8scluster/123"}},
    #                 "label": {"name": "def", "description": "my cluster"},
    #                 "k8s_version": "1.17.0",
    #                 "pod_network_range": "10.192.0.0/12",
    #                 "service_network_range": "10.96.0.0/12",
    #                 "pod_dns_domain": "cluster.local",
    #                 "created_by_user_id": "/api/v1/user/5",
    #                 "created_by_user_name": "admin",
    #                 "created_time": 1588260014,
    #                 "k8shosts_config": [
    #                     {"node": "/api/v2/worker/k8shost/4", "role": "worker"},
    #                     {"node": "/api/v2/worker/k8shost/5", "role": "master"},
    #                 ],
    #                 "status": "ready",
    #                 "status_message": "really ready",
    #                 "api_endpoint_access": "api:1234",
    #                 "dashboard_endpoint_access": "dashboard:1234",
    #                 "admin_kube_config": "xyz==",
    #                 "dashboard_token": "abc==",
    #                 "persistent_storage": {"nimble_csi": False},
    #             },
    #             status_code=200,
    #             headers={},
    #         )
    #     if (
    #         args[0]
    #         == "https://127.0.0.1:8080/api/v2/k8scluster/123?setup_log=true"
    #     ):
    #         return MockResponse(
    #             json_data={
    #                 "_links": {"self": {"href": "/api/v2/k8scluster/123"}},
    #                 "label": {"name": "def", "description": "my cluster"},
    #                 "k8s_version": "1.17.0",
    #                 "pod_network_range": "10.192.0.0/12",
    #                 "service_network_range": "10.96.0.0/12",
    #                 "pod_dns_domain": "cluster.local",
    #                 "created_by_user_id": "/api/v1/user/5",
    #                 "created_by_user_name": "admin",
    #                 "created_time": 1588260014,
    #                 "k8shosts_config": [
    #                     {"node": "/api/v2/worker/k8shost/4", "role": "worker"},
    #                     {"node": "/api/v2/worker/k8shost/5", "role": "master"},
    #                 ],
    #                 "status": "ready",
    #                 "status_message": "really ready",
    #                 "api_endpoint_access": "api:1234",
    #                 "dashboard_endpoint_access": "dashboard:1234",
    #                 "admin_kube_config": "xyz==",
    #                 "dashboard_token": "abc==",
    #                 "persistent_storage": {"nimble_csi": False},
    #             },
    #             status_code=200,
    #             headers={},
    #         )
    #     if args[0] == "https://127.0.0.1:8080/api/v2/k8scluster/999":
    #         return MockResponse(
    #             json_data={},
    #             status_code=404,
    #             raise_for_status_flag=True,
    #             headers={},
    #         )
    #     if (
    #         args[0]
    #         == "https://127.0.0.1:8080/api/v2/k8scluster/999?setup_log=true"
    #     ):
    #         return MockResponse(
    #             json_data={},
    #             status_code=404,
    #             raise_for_status_flag=True,
    #             headers={},
    #         )
    #     raise RuntimeError("Unhandle GET request: " + args[0])

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_k8scluster_assertions(self, mock_get, mock_post):

        # FIXME speed these tests up

        with self.assertRaisesRegexp(
            AssertionError, "'id' must be provided and must be a str"
        ):
            get_client().k8s_cluster.wait_for_status(
                id=1,
                timeout_secs=1,
                status=[K8sClusterStatus.ready],
            )

        # pylint: disable=anomalous-backslash-in-string
        with self.assertRaisesRegexp(
            AssertionError,
            "'id' does not start with '/api/v2/k8scluster'",
        ):
            get_client().k8s_cluster.wait_for_status(
                id="garbage",
                timeout_secs=1,
                status=[K8sClusterStatus.ready],
            )

        with self.assertRaisesRegexp(
            AssertionError, "'timeout_secs' must be an int"
        ):
            get_client().k8s_cluster.wait_for_status(
                id="/api/v2/k8scluster/123",
                timeout_secs="blah",
                status=[K8sClusterStatus.ready],
            )

        with self.assertRaisesRegexp(
            AssertionError, "'timeout_secs' must be >= 0"
        ):
            get_client().k8s_cluster.wait_for_status(
                id="/api/v2/k8scluster/123",
                timeout_secs=-1,
                status=[K8sClusterStatus.ready],
            )

        with self.assertRaisesRegexp(
            AssertionError,
            "'status' item '0' is not of type <enum 'K8sClusterStatus'>",
        ):
            get_client().k8s_cluster.wait_for_status(
                id="/api/v2/k8scluster/123",
                timeout_secs=1,
                status=["abc"],
            )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_k8scluster_body(self, mock_get, mock_post):

        self.assertTrue(
            get_client().k8s_cluster.wait_for_status(
                id="/api/v2/k8scluster/123",
                timeout_secs=1,
                status=[K8sClusterStatus.ready],
            )
        )

        self.assertFalse(
            get_client().k8s_cluster.wait_for_status(
                id="/api/v2/k8scluster/123",
                timeout_secs=1,
                status=[K8sClusterStatus.updating],
            )
        )

        self.assertTrue(
            get_client().k8s_cluster.wait_for_status(
                id="/api/v2/k8scluster/123",
                timeout_secs=1,
                status=[K8sClusterStatus.ready, K8sClusterStatus.upgrading],
            )
        )

        self.assertFalse(
            get_client().k8s_cluster.wait_for_status(
                id="/api/v2/k8scluster/123",
                timeout_secs=1,
                status=[K8sClusterStatus.warning, K8sClusterStatus.upgrading],
            )
        )

        # Get the status of a Cluster ID that doesn't exist
        with self.assertRaises(APIItemNotFoundException):
            get_client().k8s_cluster.wait_for_status(
                id="/api/v2/k8scluster/999",
                timeout_secs=1,
                status=[K8sClusterStatus.ready],
            )

        # Get the status of a Cluster ID that doesn't exist
        # without providing a status
        self.assertTrue(
            get_client().k8s_cluster.wait_for_status(
                id="/api/v2/k8scluster/999",
                timeout_secs=1,
                status=[],
            )
        )


class TestDeleteCluster(BaseTestCase):

    non_existent_cluster_url = "/api/v2/k8scluster/999"

    existing_cluster_url = "/api/v2/k8scluster/123"

    # pylint: disable=no-method-argument

    @patch("requests.delete", side_effect=BaseTestCase.httpDeleteHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_delete_k8scluster(self, mock_get, mock_post):

        # pylint: disable=anomalous-backslash-in-string
        with self.assertRaisesRegexp(
            AssertionError,
            ("'id' does not start with '/api/v2/k8scluster'"),
        ):
            get_client().k8s_cluster.delete(id="garbage")

        with self.assertRaises(APIItemNotFoundException):
            get_client().k8s_cluster.delete(
                id=TestDeleteCluster.non_existent_cluster_url
            )

        get_client().k8s_cluster.delete(
            id=TestDeleteCluster.existing_cluster_url
        )

    @patch("requests.delete", side_effect=BaseTestCase.httpDeleteHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_delete_k8scluster_cli(self, mock_delete, mock_get):

        try:
            hpecp = self.cli.CLI()
            hpecp.k8scluster.delete(id=TestDeleteCluster.existing_cluster_url)

            self.maxDiff = None

            output = self.out.getvalue().strip()
            self.assertEqual(output, "")

            # FIXME: This is failing on Python 3x because stderr seems to contain
            #        the unitest framework output.

            # error = self.err.getvalue().strip()
            # self.assertEqual(error, "")

        except Exception:
            self.fail("Unexpected exception.")

    @patch("requests.delete", side_effect=BaseTestCase.httpDeleteHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_delete_k8scluster_cli_with_exception(self, mock_delete, mock_get):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.k8scluster.delete(
                id=TestDeleteCluster.non_existent_cluster_url
            )

            self.maxDiff = None

            output = self.out.getvalue().strip()
            self.assertEqual(output, "")

            error = self.err.getvalue().strip()
            self.assertEqual(error, "'/api/v2/k8scluster/999' does not exist")

            self.assertEqual(cm.exception.code, 1)


class TestK8sSupportVersions(BaseTestCase):

    # # pylint: disable=no-method-argument
    # def mocked_requests_get(*args, **kwargs):
    #     if args[0] == "https://127.0.0.1:8080/api/v2/k8smanifest":
    #         return MockResponse(
    #             json_data={
    #                 "_version": "1.0",
    #                 "supported_versions": [
    #                     "1.14.10",
    #                     "1.15.7",
    #                     "1.16.4",
    #                     "1.17.0",
    #                     "1.18.0",
    #                 ],
    #                 "version_info": {
    #                     "1.14.10": {
    #                         "_version": "1.0",
    #                         "min_upgrade_version": "1.13.0",
    #                         "relnote_url": (
    #                             "https://v1-14.docs.kubernetes.io/docs/setup"
    #                             "/release/notes/"
    #                         ),
    #                         "hpecsi": "1.14",
    #                     },
    #                     "1.15.7": {
    #                         "_version": "1.0",
    #                         "min_upgrade_version": "1.14.0",
    #                         "relnote_url": (
    #                             "https://v1-15.docs.kubernetes.io/docs/setup"
    #                             "/release/notes/"
    #                         ),
    #                         "hpecsi": "1.15",
    #                     },
    #                     "1.16.4": {
    #                         "_version": "1.0",
    #                         "min_upgrade_version": "1.15.0",
    #                         "relnote_url": (
    #                             "https://v1-16.docs.kubernetes.io/docs/setup"
    #                             "/release/notes/"
    #                         ),
    #                         "hpecsi": "1.16",
    #                     },
    #                     "1.17.0": {
    #                         "_version": "1.0",
    #                         "min_upgrade_version": "1.16.0",
    #                         "relnote_url": (
    #                             "https://v1-17.docs.kubernetes.io/docs/setup"
    #                             "/release/notes/"
    #                         ),
    #                         "hpecsi": "1.17",
    #                     },
    #                     "1.18.0": {
    #                         "_version": "1.0",
    #                         "min_upgrade_version": "1.17.0",
    #                         "relnote_url": (
    #                             "https://kubernetes.io/docs/setup"
    #                             "/release/notes/"
    #                         ),
    #                         "hpecsi": "1.18",
    #                     },
    #                 },
    #             },
    #             status_code=200,
    #             headers={},
    #         )
    #     raise RuntimeError("Unhandle GET request: " + args[0])

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_k8s_supported_versions(self, mock_get, mock_post):

        self.assertEquals(
            get_client().k8s_cluster.k8s_supported_versions(),
            ["1.14.10", "1.15.7", "1.16.4", "1.17.0", "1.17.1", "1.18.0"],
        )


class TestCLI(BaseTestCase):

    # pylint: disable=no-method-argument
    # def mocked_requests_get(*args, **kwargs):
    # if args[0] == "https://127.0.0.1:8080/api/v2/k8scluster":
    #     return MockResponse(
    #         json_data={
    #             "_links": {"self": {"href": "/api/v2/k8scluster"}},
    #             "_embedded": {
    #                 "k8sclusters": [
    #                     {
    #                         "_links": {
    #                             "self": {"href": "/api/v2/k8scluster/20"}
    #                         },
    #                         "label": {
    #                             "name": "def",
    #                             "description": "my cluster",
    #                         },
    #                         "k8s_version": "1.17.0",
    #                         "pod_network_range": "10.192.0.0/12",
    #                         "service_network_range": "10.96.0.0/12",
    #                         "pod_dns_domain": "cluster.local",
    #                         "created_by_user_id": "/api/v1/user/5",
    #                         "created_by_user_name": "admin",
    #                         "created_time": 1588260014,
    #                         "k8shosts_config": [
    #                             {
    #                                 "node": "/api/v2/worker/k8shost/4",
    #                                 "role": "worker",
    #                             },
    #                             {
    #                                 "node": "/api/v2/worker/k8shost/5",
    #                                 "role": "master",
    #                             },
    #                         ],
    #                         "status": "ready",
    #                         "status_message": "really ready",
    #                         "api_endpoint_access": "api:1234",
    #                         "dashboard_endpoint_access": "dashboard:1234",
    #                         "admin_kube_config": "xyz==",
    #                         "dashboard_token": "abc==",
    #                         "persistent_storage": {"nimble_csi": False},
    #                     }
    #                 ]
    #             },
    #         },
    #         status_code=200,
    #         headers={},
    #     )
    # if args[0] == "https://127.0.0.1:8080/api/v2/k8smanifest":
    #     return MockResponse(
    #         json_data={
    #             "_version": "1.0",
    #             "supported_versions": [
    #                 "1.14.10",
    #                 "1.15.7",
    #                 "1.16.4",
    #                 "1.17.0",
    #                 "1.17.1",
    #                 "1.18.0",
    #             ],
    #             "version_info": {
    #                 "1.14.10": {
    #                     "_version": "1.0",
    #                     "min_upgrade_version": "1.13.0",
    #                     "relnote_url": (
    #                         "https://v1-14.docs.kubernetes.io/docs/setup"
    #                         "/release/notes/"
    #                     ),
    #                     "hpecsi": "1.14",
    #                 },
    #                 "1.15.7": {
    #                     "_version": "1.0",
    #                     "min_upgrade_version": "1.14.0",
    #                     "relnote_url": (
    #                         "https://v1-15.docs.kubernetes.io/docs/setup"
    #                         "/release/notes/"
    #                     ),
    #                     "hpecsi": "1.15",
    #                 },
    #                 "1.16.4": {
    #                     "_version": "1.0",
    #                     "min_upgrade_version": "1.15.0",
    #                     "relnote_url": (
    #                         "https://v1-16.docs.kubernetes.io/docs/setup"
    #                         "/release/notes/"
    #                     ),
    #                     "hpecsi": "1.16",
    #                 },
    #                 "1.17.0": {
    #                     "_version": "1.0",
    #                     "min_upgrade_version": "1.16.0",
    #                     "relnote_url": (
    #                         "https://v1-17.docs.kubernetes.io/docs/setup"
    #                         "/release/notes/"
    #                     ),
    #                     "hpecsi": "1.17",
    #                 },
    #                 "1.17.1": {
    #                     "_version": "1.0",
    #                     "min_upgrade_version": "1.17.0",
    #                     "relnote_url": (
    #                         "https://v1-17.docs.kubernetes.io/docs/setup"
    #                         "/release/notes/"
    #                     ),
    #                     "hpecsi": "1.17",
    #                 },
    #                 "1.18.0": {
    #                     "_version": "1.0",
    #                     "min_upgrade_version": "1.17.0",
    #                     "relnote_url": (
    #                         "https://kubernetes.io/docs/setup"
    #                         "/release/notes/"
    #                     ),
    #                     "hpecsi": "1.18",
    #                 },
    #             },
    #         },
    #         status_code=200,
    #         headers={},
    #     )
    # raise RuntimeError("Unhandle GET request: " + args[0])

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8scluster_list(self, mock_post, mock_get):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.list(columns=["id", "name", "description", "status"])

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            (
                "+-----------------------+------+-------------+--------+\n"
                "|          id           | name | description | status |\n"
                "+-----------------------+------+-------------+--------+\n"
                "| /api/v2/k8scluster/20 | def  | my cluster  | ready  |\n"
                "+-----------------------+------+-------------+--------+"
            ),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_no_filter(self, mock_post, mock_get):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.k8s_supported_versions()

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "['1.14.10', '1.15.7', '1.16.4', '1.17.0', '1.17.1', '1.18.0']",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_no_filter_output_json(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.k8s_supported_versions(output="json")

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "['1.14.10', '1.15.7', '1.16.4', '1.17.0', '1.17.1', '1.18.0']",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_no_filter_output_text(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.k8s_supported_versions(output="text")

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "1.14.10 1.15.7 1.16.4 1.17.0 1.17.1 1.18.0",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_no_filter_output_invalid(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()

        with self.assertRaises(SystemExit) as cm:
            hpecp.k8scluster.k8s_supported_versions(output="garbage")

        self.assertEqual(cm.exception.code, 1)

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "",
        )

        error = self.err.getvalue().strip()
        self.assertEqual(
            error,
            "'output' parameter ust be 'json' or 'text'",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_major_filter_match(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.k8s_supported_versions(major_filter=1)

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "['1.14.10', '1.15.7', '1.16.4', '1.17.0', '1.17.1', '1.18.0']",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_major_filter_no_match(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.k8s_supported_versions(major_filter=2)

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "[]",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_minor_filter_match(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.k8s_supported_versions(minor_filter=17)

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "['1.17.0', '1.17.1']",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_minor_filter_no_match(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.k8s_supported_versions(minor_filter=2)

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "[]",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_patch_filter_match(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.k8s_supported_versions(patch_filter=0)

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "['1.17.0', '1.18.0']",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_patch_filter_no_match(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.k8s_supported_versions(patch_filter=21)

        output = self.out.getvalue().strip()
        self.assertEqual(
            output,
            "[]",
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_major_filter_invalid(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        with self.assertRaises(SystemExit) as cm:
            hpecp.k8scluster.k8s_supported_versions(major_filter="text")

        output = self.out.getvalue().strip()
        self.assertEqual(output, "")

        error = self.err.getvalue().strip()
        self.assertEqual(error, "'major_filter' if provided must be an int")

        self.assertEqual(cm.exception.code, 1)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_minor_filter_invalid(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        with self.assertRaises(SystemExit) as cm:
            hpecp.k8scluster.k8s_supported_versions(minor_filter="text")

        output = self.out.getvalue().strip()
        self.assertEqual(output, "")

        error = self.err.getvalue().strip()
        self.assertEqual(error, "'minor_filter' if provided must be an int")

        self.assertEqual(cm.exception.code, 1)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_k8s_supported_verions_patch_filter_invalid(
        self, mock_post, mock_get
    ):

        hpecp = self.cli.CLI()
        with self.assertRaises(SystemExit) as cm:
            hpecp.k8scluster.k8s_supported_versions(patch_filter="text")

        output = self.out.getvalue().strip()
        self.assertEqual(output, "")

        error = self.err.getvalue().strip()
        self.assertEqual(error, "'patch_filter' if provided must be an int")

        self.assertEqual(cm.exception.code, 1)

    def mocked_requests_create_post(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/login":
            return MockResponse(
                json_data={},
                status_code=200,
                headers={
                    "location": (
                        "/api/v1/session/df1bfacb-xxxx-xxxx-xxxx-c8f57d8f3c71"
                    )
                },
            )
        elif args[0] == "https://127.0.0.1:8080/api/v2/k8scluster":
            return MockResponse(
                json_data={},
                status_code=200,
                headers={"Location": "/api/v2/k8sclusters/99"},
            )
        raise RuntimeError("Unhandle POST request: " + args[0])

    @patch("requests.post", side_effect=mocked_requests_create_post)
    def test_k8scluster_create(self, mock_post):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.create(
            name="mycluster",
            k8shosts_config="/api/v2/worker/k8shost/1:master,/api/v2/worker/k8shost/2:worker",
        )

        output = self.out.getvalue().strip()
        self.assertEqual(output, "/api/v2/k8sclusters/99")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_k8scluster_admin_kube_config(self, mock_get, mock_post):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.admin_kube_config(id="/api/v2/k8scluster/123")

        output = self.out.getvalue().strip()
        self.assertEqual(output, "test_admin_kube_config")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_k8scluster_dashboard_url(self, mock_get, mock_post):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.dashboard_url(id="/api/v2/k8scluster/123")

        output = self.out.getvalue().strip()
        self.assertEqual(output, "test_dashboard_url")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_k8scluster_dashboard_token(self, mock_get, mock_post):

        hpecp = self.cli.CLI()
        hpecp.k8scluster.dashboard_token(id="/api/v2/k8scluster/123")

        output = self.out.getvalue().strip()
        self.assertEqual(output, "abc")  # abc base64 encoded is: YWJjCg==


class TestCliStates(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_states(self, mock_post):

        self.maxDiff = None

        hpecp = self.cli.CLI()
        hpecp.k8scluster.statuses()
        stdout = self.out.getvalue().strip()

        expected_stdout = str(
            [
                "ready",
                "creating",
                "updating",
                "upgrading",
                "deleting",
                "error",
                "warning",
            ]
        )

        self.assertEqual(stdout, expected_stdout)


class TestK8sClusterHostConfig(TestCase):
    def test_cluster_host_config(self):

        expected_error = "'noderole' list must have two values [ node, role ]"
        try:
            K8sClusterHostConfig.create_from_list(noderole=[1, 2, 3])
        except AssertionError as e:
            self.assertEquals(
                e.args[0],
                expected_error,
            )

        conf = K8sClusterHostConfig.create_from_list(
            noderole=["/api/v2/worker/k8shost/1", "master"]
        )
        self.assertIsInstance(conf, K8sClusterHostConfig)



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from unittest import TestCase

import six
from mock import patch

from hpecp import ContainerPlatformClient
from hpecp.exceptions import APIItemNotFoundException

from .base import BaseTestCase, get_client
from .user_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestUsers(TestCase):
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_users(self, mock_get, mock_post):
        client = ContainerPlatformClient(
            username="admin",
            password="admin123",
            api_host="127.0.0.1",
            api_port=8080,
            use_ssl=True,
        )

        # Makes POST Request: https://127.0.0.1:8080/api/v1/login
        client.create_session()

        # Makes GET Request: https://127.0.0.1:8080/api/v1/user
        users = client.user.list()

        # Test that json response is saved in each WorkerK8s object
        assert users[0].json is not None

        # Test UserList subscriptable access and property setters
        assert users[0].is_service_account is False
        assert users[0].is_siteadmin is False
        assert users[0].default_tenant == ""
        assert users[0].is_external is False
        assert users[0].is_group_added_user is False

        assert users[0].name == "csnow"
        assert users[0].description == "chris"
        assert users[0]._links == {"self": {"href": "/api/v1/user/16"}}


class TestDeleteUser(TestCase):
    @patch("requests.delete", side_effect=BaseTestCase.httpDeleteHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_delete_user(self, mock_post, mock_delete):
        with self.assertRaisesRegexp(
            AssertionError,
            "'id' must be provided and must be a str",
        ):
            get_client().user.delete(id=999)

        with self.assertRaisesRegexp(
            AssertionError,
            "'id' does not start with '/api/v1/user/'",
        ):
            get_client().user.delete(id="garbage")

        with self.assertRaises(APIItemNotFoundException):
            get_client().user.delete(id="/api/v1/user/999")

        get_client().user.delete(id="/api/v1/user/123")


class TestCLI(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_create(self, mock_post):

        hpecp = self.cli.CLI()
        hpecp.user.create(
            name="jdoe",
            password="secret",
            description="Jane Doe",
            is_external=False,
        )

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = "/mock/api/user/1"
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


from .base import BaseTestCase, MockResponse


def mockApiSetup():

    # Response for cluster list all
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/k8scluster",
        response=MockResponse(
            json_data={
                "_links": {"self": {"href": "/api/v2/k8scluster"}},
                "_embedded": {
                    "k8sclusters": [
                        {
                            "_links": {
                                "self": {"href": "/api/v2/k8scluster/20"}
                            },
                            "label": {
                                "name": "def",
                                "description": "my cluster",
                            },
                            "k8s_version": "1.17.0",
                            "pod_network_range": "10.192.0.0/12",
                            "service_network_range": "10.96.0.0/12",
                            "pod_dns_domain": "cluster.local",
                            "created_by_user_id": "/api/v1/user/5",
                            "created_by_user_name": "admin",
                            "created_time": 1588260014,
                            "k8shosts_config": [
                                {
                                    "node": "/api/v2/worker/k8shost/4",
                                    "role": "worker",
                                },
                                {
                                    "node": "/api/v2/worker/k8shost/5",
                                    "role": "master",
                                },
                            ],
                            "status": "ready",
                            "status_message": "really ready",
                            "api_endpoint_access": "api:1234",
                            "dashboard_endpoint_access": "dashboard:1234",
                            "admin_kube_config": "xyz==",
                            "dashboard_token": "abc==",
                            "persistent_storage": {"nimble_csi": False},
                        }
                    ]
                },
            },
            status_code=200,
            headers={},
        ),
    )

    # Response for a cluster that doesn't exist
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/k8scluster/999",
        response=MockResponse(
            json_data={},
            status_code=404,
            raise_for_status_flag=True,
            headers={},
        ),
    )

    # Response for a cluster that doesn't exist
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/k8scluster/999?setup_log=true",
        response=MockResponse(
            json_data={},
            status_code=404,
            raise_for_status_flag=True,
            headers={},
        ),
    )

    # Response for a cluster that DOES exist
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/k8scluster/123",
        response=MockResponse(
            json_data={
                "_links": {"self": {"href": "/api/v2/k8scluster/123"}},
                "label": {"name": "def", "description": "my cluster"},
                "k8s_version": "1.17.0",
                "pod_network_range": "10.192.0.0/12",
                "service_network_range": "10.96.0.0/12",
                "pod_dns_domain": "cluster.local",
                "created_by_user_id": "/api/v1/user/5",
                "created_by_user_name": "admin",
                "created_time": 1588260014,
                "k8shosts_config": [
                    {"node": "/api/v2/worker/k8shost/4", "role": "worker"},
                    {"node": "/api/v2/worker/k8shost/5", "role": "master"},
                ],
                "status": "ready",
                "status_message": "really ready",
                "api_endpoint_access": "api:1234",
                "dashboard_endpoint_access": "test_dashboard_url",
                "admin_kube_config": "test_admin_kube_config",
                "dashboard_token": "YWJjCg==",
                "persistent_storage": {"nimble_csi": False},
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/k8scluster/123?setup_log=true",
        response=MockResponse(
            json_data={
                "_links": {"self": {"href": "/api/v2/k8scluster/123"}},
                "label": {"name": "def", "description": "my cluster"},
                "k8s_version": "1.17.0",
                "pod_network_range": "10.192.0.0/12",
                "service_network_range": "10.96.0.0/12",
                "pod_dns_domain": "cluster.local",
                "created_by_user_id": "/api/v1/user/5",
                "created_by_user_name": "admin",
                "created_time": 1588260014,
                "k8shosts_config": [
                    {"node": "/api/v2/worker/k8shost/4", "role": "worker"},
                    {"node": "/api/v2/worker/k8shost/5", "role": "master"},
                ],
                "status": "ready",
                "status_message": "really ready",
                "api_endpoint_access": "api:1234",
                "dashboard_endpoint_access": "dashboard:1234",
                "admin_kube_config": "xyz==",
                "dashboard_token": "abc==",
                "persistent_storage": {"nimble_csi": False},
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/k8smanifest",
        response=MockResponse(
            json_data={
                "_version": "1.0",
                "supported_versions": [
                    "1.14.10",
                    "1.15.7",
                    "1.16.4",
                    "1.17.0",
                    "1.17.1",
                    "1.18.0",
                ],
                "version_info": {
                    "1.14.10": {
                        "_version": "1.0",
                        "min_upgrade_version": "1.13.0",
                        "relnote_url": (
                            "https://v1-14.docs.kubernetes.io/docs/setup"
                            "/release/notes/"
                        ),
                        "hpecsi": "1.14",
                    },
                    "1.15.7": {
                        "_version": "1.0",
                        "min_upgrade_version": "1.14.0",
                        "relnote_url": (
                            "https://v1-15.docs.kubernetes.io/docs/setup"
                            "/release/notes/"
                        ),
                        "hpecsi": "1.15",
                    },
                    "1.16.4": {
                        "_version": "1.0",
                        "min_upgrade_version": "1.15.0",
                        "relnote_url": (
                            "https://v1-16.docs.kubernetes.io/docs/setup"
                            "/release/notes/"
                        ),
                        "hpecsi": "1.16",
                    },
                    "1.17.0": {
                        "_version": "1.0",
                        "min_upgrade_version": "1.16.0",
                        "relnote_url": (
                            "https://v1-17.docs.kubernetes.io/docs/setup"
                            "/release/notes/"
                        ),
                        "hpecsi": "1.17",
                    },
                    "1.17.1": {
                        "_version": "1.0",
                        "min_upgrade_version": "1.17.0",
                        "relnote_url": (
                            "https://v1-17.docs.kubernetes.io/docs/setup"
                            "/release/notes/"
                        ),
                        "hpecsi": "1.17",
                    },
                    "1.18.0": {
                        "_version": "1.0",
                        "min_upgrade_version": "1.17.0",
                        "relnote_url": (
                            "https://kubernetes.io/docs/setup"
                            "/release/notes/"
                        ),
                        "hpecsi": "1.18",
                    },
                },
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpPostHandler(
        url="https://127.0.0.1:8080/api/v2/k8scluster",
        response=MockResponse(
            json_data={},
            status_code=200,
            headers={"Location": "/api/v2/k8scluster/99"},
        ),
    )

    BaseTestCase.registerHttpDeleteHandler(
        url="https://127.0.0.1:8080/api/v2/k8scluster/999",
        response=MockResponse(
            json_data={},
            status_code=404,
            raise_for_status_flag=True,
            headers={},
        ),
    )

    BaseTestCase.registerHttpDeleteHandler(
        url="https://127.0.0.1:8080/api/v2/k8scluster/123",
        response=MockResponse(
            json_data={
                "_links": {"self": {"href": "/api/v2/k8scluster/123"}},
                "label": {"name": "def", "description": "my cluster"},
                "k8s_version": "1.17.0",
                "pod_network_range": "10.192.0.0/12",
                "service_network_range": "10.96.0.0/12",
                "pod_dns_domain": "cluster.local",
                "created_by_user_id": "/api/v1/user/5",
                "created_by_user_name": "admin",
                "created_time": 1588260014,
                "k8shosts_config": [
                    {"node": "/api/v2/worker/k8shost/4", "role": "worker"},
                    {"node": "/api/v2/worker/k8shost/5", "role": "master"},
                ],
                "status": "ready",
                "status_message": "really ready",
                "api_endpoint_access": "api:1234",
                "dashboard_endpoint_access": "dashboard:1234",
                "admin_kube_config": "xyz==",
                "dashboard_token": "abc==",
                "persistent_storage": {"nimble_csi": False},
            },
            status_code=200,
            headers={},
        ),
    )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import abc
import os
import sys
import tempfile
import unittest
from io import StringIO
from textwrap import dedent

import requests
import six

from hpecp import ContainerPlatformClient

if six.PY2:
    from io import BytesIO as StringIO  # noqa: F811
else:
    from io import StringIO


class MockResponse:
    def __init__(
        self,
        json_data,
        status_code,
        headers,
        raise_for_status_flag=False,
        raise_connection_error=False,
        text_data="",
    ):
        self.json_data = json_data
        self.text = text_data
        self.status_code = status_code
        self.raise_for_status_flag = raise_for_status_flag
        self.raise_connection_error = raise_connection_error
        self.headers = headers

    def raise_for_status(self):
        if self.raise_for_status_flag:
            self.text = "some error occurred"
            raise requests.exceptions.HTTPError()
        if self.raise_connection_error:
            self.text = "Simulating a connection error"
            raise requests.exceptions.ConnectionError()
        else:
            return

    def json(self):
        return self.json_data


def get_client():
    client = ContainerPlatformClient(
        username="admin",
        password="admin123",
        api_host="127.0.0.1",
        api_port=8080,
        use_ssl=True,
    )
    client.create_session()
    return client


@six.add_metaclass(abc.ABCMeta)
class BaseTestCase(unittest.TestCase):

    _http_get_handlers = {}
    _http_post_handlers = {}
    _http_put_handlers = {}
    _http_delete_handlers = {}

    @classmethod
    def httpPostHandlers(cls, *args, **kwargs):
        try:
            handler = BaseTestCase._http_post_handlers[args[0]]
        except KeyError:
            raise Exception(
                "Handler not found for POST '{}'.\nDid you register a handler with BaseTestCase.registerHttpPostHandler?".format(
                    args[0]
                )
            )

        if isinstance(handler, Exception):
            raise handler
        else:
            return handler

    @classmethod
    def httpPutHandlers(cls, *args, **kwargs):
        try:
            handler = BaseTestCase._http_put_handlers[args[0]]
        except KeyError:
            raise Exception(
                "Handler not found for PUT '{}'.\nDid you register a handler with BaseTestCase.registerHttpPutHandler?".format(
                    args[0]
                )
            )

        if isinstance(handler, Exception):
            raise handler
        else:
            return handler

    @classmethod
    def httpGetHandlers(cls, *args, **kwargs):
        try:
            handler = BaseTestCase._http_get_handlers[args[0]]
        except KeyError:
            raise Exception(
                "Handler not found for GET '{}'.\nDid you register a handler with BaseTestCase.registerHttpGetHandler?".format(
                    args[0]
                )
            )

        if isinstance(handler, Exception):
            raise handler
        else:
            return handler

    @classmethod
    def httpDeleteHandlers(cls, *args, **kwargs):
        try:
            handler = BaseTestCase._http_delete_handlers[args[0]]
        except KeyError:
            raise Exception(
                "Handler not found for DELETE '{}'.\nDid you register a handler with BaseTestCase.registerHttpDeleteHandler?".format(
                    args[0]
                )
            )

        if isinstance(handler, Exception):
            raise handler
        else:
            return handler

    @classmethod
    def registerHttpPostHandler(cls, url, response):
        BaseTestCase._http_post_handlers[url] = response

    @classmethod
    def registerHttpPutHandler(cls, url, response):
        BaseTestCase._http_put_handlers[url] = response

    @classmethod
    def registerHttpGetHandler(cls, url, response):
        BaseTestCase._http_get_handlers[url] = response

    @classmethod
    def registerHttpDeleteHandler(cls, url, response):
        BaseTestCase._http_delete_handlers[url] = response

    @classmethod
    def setUpClass(cls):

        _http_get_handlers = {}  # noqa: F841
        _http_post_handlers = {}  # noqa: F841
        _http_put_handlers = {}  # noqa: F841
        _http_delete_handlers = {}  # noqa: F841

        # Register the login handler
        BaseTestCase.registerHttpPostHandler(
            "https://127.0.0.1:8080/api/v1/login",
            MockResponse(
                json_data={},
                status_code=200,
                headers={
                    "location": (
                        "/api/v1/session/df1bfacb-xxxx-xxxx-xxxx-c8f57d8f3c71"
                    )
                },
            ),
        )

    def setUp(self):
        file_data = dedent(
            """[default]
                        api_host = 127.0.0.1
                        api_port = 8080
                        use_ssl = True
                        verify_ssl = False
                        warn_ssl = True
                        username = admin
                        password = admin123"""
        )

        self.tmpFile = tempfile.NamedTemporaryFile(delete=True)
        self.tmpFile.write(file_data.encode("utf-8"))
        self.tmpFile.flush()

        self.saved_stdout = sys.stdout
        self.out = StringIO()
        sys.stdout = self.out

        self.saved_stderr = sys.stderr
        self.err = StringIO()
        sys.stderr = self.err

        from hpecp.cli import base

        sys.path.insert(0, os.path.abspath("../../"))
        from bin import cli

        try:
            reload
        except NameError:
            # Python 3
            from imp import reload

        reload(cli)

        self.cli = cli

        self.saved_base_get_config_file = base.get_config_file
        self.saved_base_get_client = base.get_client

        def get_config_file():
            return self.tmpFile.name

        # override method to return config file path
        base.get_config_file = get_config_file

    def tearDown(self):
        from hpecp.cli import base

        self.tmpFile.close()
        sys.stdout = self.saved_stdout
        sys.stderr = self.saved_stderr

        base.get_config_file = self.saved_base_get_config_file
        base.get_client = self.saved_base_get_client



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest

from hpecp.base_resource import (
    AbstractResourceController,
    AbstractWaitableResourceController,
)
from hpecp.client import ContainerPlatformClient


class TestBaseResource(unittest.TestCase):
    def test_getters_and_setters(self):
        client = ContainerPlatformClient(
            username="test",
            password="test",
            api_host="test",
            api_port=8080,
            use_ssl=True,
            verify_ssl=True,
            warn_ssl=True,
        )

        class ImplClass(AbstractResourceController):
            base_resource_path = "abc"
            resource_class = "abc"
            resource_list_path = "abc"

        c = ImplClass(client)

        c._set_base_resource_path("test_base_resource_path")
        c._set_resource_class("test_resource_class")
        c._set_resource_list_path("test_resource_list_path")

        self.assertEqual(
            c._get_base_resource_path(), "test_base_resource_path"
        )
        self.assertEqual(c._get_resource_class(), "test_resource_class")
        self.assertEqual(
            c._get_resource_list_path(), "test_resource_list_path"
        )

        class ImplWaitClass(AbstractWaitableResourceController):
            status_class = "abc"
            status_fieldname = "abc"
            base_resource_path = "abc"
            resource_class = "abc"
            resource_list_path = "abc"

        c = ImplWaitClass(client)

        c._set_status_class("test_status_class")
        c._set_status_fieldname("test_status_fieldname")

        self.assertEqual(c._get_status_class(), "test_status_class")
        self.assertEqual(c._get_status_fieldname(), "test_status_fieldname")



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os
import tempfile
from textwrap import dedent
from unittest import TestCase

import requests
from mock import patch

from hpecp import ContainerPlatformClient, ContainerPlatformClientException

from .base import BaseTestCase, MockResponse
from .client_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestCreateFromProperties(TestCase):
    def test_create_from_config_file_factory_method(self):
        file_data = dedent(
            """[default]
                              api_host = 127.0.0.1
                              api_port = 8080
                              use_ssl = True
                              verify_ssl = False
                              warn_ssl = True
                              username = admin
                              password = admin123"""
        )

        tmp = tempfile.NamedTemporaryFile(delete=True)
        try:
            tmp.write(file_data.encode("utf-8"))
            tmp.flush()

            client = ContainerPlatformClient.create_from_config_file(
                config_file=tmp.name
            )
            self.assertEqual(client.username, "admin")
            self.assertEqual(client.password, "admin123")
            self.assertEqual(client.api_host, "127.0.0.1")
            self.assertEqual(client.api_port, 8080)
            self.assertEqual(client.use_ssl, True)
            self.assertEqual(client.verify_ssl, False)
            self.assertEqual(client.warn_ssl, True)
        finally:
            tmp.close()


class TestCreateFromEnvVar(TestCase):
    @patch.dict(os.environ, {"HPECP_USERNAME": "test_username"})
    def test_create_from_env_var_factory_method_with_missing_env_values(self):

        try:
            ContainerPlatformClient.create_from_env()
        except ContainerPlatformClientException as expected:
            self.assertEqual(
                expected.message,
                "Required env var 'HPECP_PASSWORD' not found.",
            )

    @patch.dict(
        os.environ,
        {
            "HPECP_USERNAME": "test_username",
            "HPECP_PASSWORD": "test_password",
            "HPECP_API_HOST": "test_apihost",
            "HPECP_API_PORT": "not_an_int",
            "HPECP_USE_SSL": "True",
            "HPECP_VERIFY_SSL": "True",
            "HPECP_WARN_SSL": "True",
        },
    )
    def test_create_from_env_var_factory_method_with_type_error(self):

        try:
            ContainerPlatformClient.create_from_env()
        except ContainerPlatformClientException as expected:
            self.assertEqual(
                expected.message,
                "invalid literal for int() with base 10: 'not_an_int'",
            )

    @patch.dict(
        os.environ,
        {
            "HPECP_USERNAME": "test_username",
            "HPECP_PASSWORD": "test_password",
            "HPECP_API_HOST": "test_apihost",
            "HPECP_API_PORT": "8080",
            "HPECP_USE_SSL": "True",
            "HPECP_VERIFY_SSL": "True",
            "HPECP_WARN_SSL": "True",
        },
    )
    def test_create_from_env_var_factory_method(self):

        client = ContainerPlatformClient.create_from_env()

        self.assertEqual(client.username, "test_username")
        self.assertEqual(client.password, "test_password")
        self.assertEqual(client.api_host, "test_apihost")
        self.assertEqual(client.api_port, 8080)
        self.assertEqual(client.use_ssl, True)
        self.assertEqual(client.verify_ssl, True)
        self.assertEqual(client.warn_ssl, True)

    @patch.dict(
        os.environ,
        {
            "HPECP_USERNAME": "test_username",
            "HPECP_PASSWORD": "test_password",
            "HPECP_API_HOST": "test_apihost",
            "HPECP_API_PORT": "8080",
            "HPECP_USE_SSL": "False",
            "HPECP_VERIFY_SSL": "False",
            "HPECP_WARN_SSL": "False",
        },
    )
    def test_create_from_env_var_factory_method_with_falses(self):

        client = ContainerPlatformClient.create_from_env()

        self.assertEqual(client.username, "test_username")
        self.assertEqual(client.password, "test_password")
        self.assertEqual(client.api_host, "test_apihost")
        self.assertEqual(client.api_port, 8080)
        self.assertEqual(client.use_ssl, False)
        self.assertEqual(client.verify_ssl, False)
        self.assertEqual(client.warn_ssl, False)


class TestAuth(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_create_session(self, mock_post):

        client = ContainerPlatformClient(
            username="admin",
            password="admin123",
            api_host="127.0.0.1",
            api_port=8080,
            use_ssl=False,
        )

        self.assertIsInstance(client.create_session(), ContainerPlatformClient)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_create_session_chained(self, mock_post):

        client = ContainerPlatformClient(
            username="admin",
            password="admin123",
            api_host="127.0.0.1",
            api_port=8080,
            use_ssl=False,
        ).create_session()

        self.assertIsInstance(client, ContainerPlatformClient)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_auth_ssl(self, mock_post):

        client = ContainerPlatformClient(
            username="admin",
            password="admin123",
            api_host="127.0.0.1",
            api_port=8080,
            use_ssl=True,
        )
        client.create_session()

    def mocked_requests_post_return_500(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/login":
            return MockResponse(
                json_data={},
                status_code=500,
                headers={},
                raise_for_status_flag=True,
            )
        raise RuntimeError("Unhandle POST request: " + args[0])

    @patch("requests.post", side_effect=mocked_requests_post_return_500)
    def test_auth_ssl_with_error(self, mock_post):

        client = ContainerPlatformClient(
            username="admin",
            password="admin123",
            api_host="127.0.0.1",
            api_port=8080,
            use_ssl=True,
        )

        with self.assertRaises(requests.exceptions.HTTPError):
            client.create_session()



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


import json
import unittest
from textwrap import dedent

import yaml
from mock import patch

from hpecp.base_resource import ResourceList
from hpecp.exceptions import APIItemNotFoundException

from .base import BaseTestCase, MockResponse, get_client
from .catalog_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestCatalogGet(BaseTestCase):
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_catalog_id_type(self, mock_get, mock_post):

        with self.assertRaisesRegexp(
            AssertionError,
            "'id' must be provided and must be a str",
        ):
            get_client().catalog.get(123)

        with self.assertRaisesRegexp(
            AssertionError,
            "'id' must be provided and must be a str",
        ):
            get_client().catalog.get(False)

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_catalog_id_format(self, mock_get, mock_post):

        with self.assertRaisesRegexp(
            AssertionError, "'id' does not start with '/api/v1/catalog'"
        ):
            get_client().catalog.get("garbage")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_catalog(self, mock_get, mock_post):

        get_client().catalog.get("/api/v1/catalog/99")

        with self.assertRaisesRegexp(
            APIItemNotFoundException,
            "'catalog not found with id: " + r"\/api\/v1\/catalog\/101",
        ):
            get_client().catalog.get("/api/v1/catalog/101")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_catalog_attributes(self, mock_get, mock_post):

        catalog = get_client().catalog.get("/api/v1/catalog/99")
        self.assertEqual(catalog.distro_id, "bluedata/spark240juphub7xssl")
        self.assertEqual(
            catalog.documentation_checksum, "52f53f1b2845463b9e370d17fb80bea6"
        )
        self.assertEqual(
            catalog.documentation_file,
            "/opt/bluedata/catalog/documentation/bluedata-spark240juphub7xssl-2.8",
        )
        self.assertEqual(catalog.documentation_mimetype, "text/markdown")
        self.assertEqual(
            catalog.feed,
            [
                {
                    "href": "https://s3.amazonaws.com/bluedata-catalog/bundles/catalog/external/docker/EPIC-5.0/feeds/feed.json",
                    "name": "BlueData EPIC-5.0 catalog feed for docker",
                }
            ],
        )
        self.assertEqual(catalog.id, "/api/v1/catalog/99")
        self.assertEqual(catalog.isdebug, False)
        self.assertEqual(
            catalog.label_description,
            "Spark240 multirole with Jupyter Notebook, Jupyterhub with SSL and gateway node",
        )
        self.assertEqual(catalog.label_name, "Spark240")
        self.assertEqual(
            catalog.logo_checksum, "1471eb59356066ed4a06130566764ea6"
        )
        self.assertEqual(
            catalog.logo_url,
            "http://10.1.0.53/catalog/logos/bluedata-spark240juphub7xssl-2.8",
        )
        self.assertEqual(catalog.osclass, ["centos"])
        self.assertEqual(catalog.self_href, "/api/v1/catalog/99")
        self.assertEqual(catalog.state, "initialized")
        self.assertEqual(catalog.state_info, "")
        self.assertEqual(catalog.timestamp, 0)
        self.assertEqual(catalog.version, "2.8")


class TestCatalogList(unittest.TestCase):
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_list(self, mock_get, mock_post):

        catalog_list = get_client().catalog.list()
        self.assertIsInstance(catalog_list, ResourceList)


class TestCatalogInstall(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_catalog_install(self, mock_get, mock_post):

        client = get_client()

        with self.assertRaisesRegexp(
            AssertionError,
            "'id' must be provided and must be a str",
        ):
            client.catalog.install(999)

        with self.assertRaisesRegexp(
            AssertionError,
            "'id' does not start with '/api/v1/catalog'",
        ):
            client.catalog.install("garbage")

        with self.assertRaisesRegexp(
            APIItemNotFoundException,
            "'catalog not found with id: /api/v1/catalog/101'",
        ):
            client.catalog.install("/api/v1/catalog/101")

        client.catalog.install("/api/v1/catalog/99")

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_catalog_install_cli_with_parameter_assertion_error(
        self, mock_get, mock_post
    ):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.catalog.install("garbage")

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""  # we don't want error output going to stdout
        expected_stderr = "'id' does not start with '/api/v1/catalog'"

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(stderr.endswith(expected_stderr))

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_catalog_install_cli_with_catalog_id_not_found(
        self, mock_get, mock_post
    ):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.catalog.install("/api/v1/catalog/101")

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""  # we don't want error output going to stdout
        expected_stderr = "catalog not found with id: /api/v1/catalog/101"

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(
            stderr.endswith(expected_stderr),
            (
                "stderr = `{}`\n"
                "stderr does not end with `{}`".format(stderr, expected_stderr)
            ),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_catalog_install_cli_success(self, mock_get, mock_post):

        try:
            hpecp = self.cli.CLI()
            hpecp.catalog.install("/api/v1/catalog/99")
        except Exception:
            self.fail("Unexpected exception")

        stdout = self.out.getvalue().strip()

        # successful refresh should not output anything
        expected_stdout = ""

        self.assertEqual(stdout, expected_stdout)


class TestCatalogRefresh(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_catalog_refresh(self, mock_get, mock_post):

        client = get_client()

        with self.assertRaisesRegexp(
            AssertionError,
            "'id' must be provided and must be a str",
        ):
            client.catalog.install(999)

        with self.assertRaisesRegexp(
            AssertionError,
            "'id' does not start with '/api/v1/catalog'",
        ):
            client.catalog.refresh("garbage")

        with self.assertRaisesRegexp(
            APIItemNotFoundException,
            "'catalog not found with id: /api/v1/catalog/101'",
        ):
            client.catalog.refresh("/api/v1/catalog/101")

        client.catalog.refresh("/api/v1/catalog/99")

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_catalog_refresh_cli_with_parameter_assertion_error(
        self, mock_get, mock_post
    ):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.catalog.refresh("garbage")

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""  # we don't want error output going to stdout
        expected_stderr = "'id' does not start with '/api/v1/catalog'"

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(stderr.endswith(expected_stderr))

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_catalog_refresh_cli_with_catalog_id_not_found(
        self, mock_get, mock_post
    ):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.catalog.refresh("/api/v1/catalog/101")

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""  # we don't want error output going to stdout
        expected_stderr = "catalog not found with id: /api/v1/catalog/101"

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(
            stderr.endswith(expected_stderr),
            (
                "stderr = `{}`\n"
                "stderr does not end with `{}`".format(stderr, expected_stderr)
            ),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_catalog_refresh_cli_success(self, mock_get, mock_post):

        try:
            hpecp = self.cli.CLI()
            hpecp.catalog.refresh("/api/v1/catalog/99")
        except Exception:
            self.fail("Unexpected exception")

        stdout = self.out.getvalue().strip()

        # successful refresh should not output anything
        expected_stdout = ""

        self.assertEqual(stdout, expected_stdout)


class TestCLIList(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list_with_columns_and_table_output(self, mock_post, mock_get):

        self.maxDiff = None

        hpecp = self.cli.CLI()
        hpecp.catalog.list(columns=["label_name", "label_description"])

        output = self.out.getvalue().strip()

        self.assertEqual(
            output,
            (
                "+------------+--------------------------------------------------------------------------------+\n"
                "| label_name |                               label_description                                |\n"
                "+------------+--------------------------------------------------------------------------------+\n"
                "|  Spark240  | Spark240 multirole with Jupyter Notebook, Jupyterhub with SSL and gateway node |\n"
                "+------------+--------------------------------------------------------------------------------+"
            ),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list_with_columns_and_text_output(self, mock_post, mock_get):

        self.maxDiff = None

        with patch.dict("os.environ", {"LOG_LEVEL": "DEBUG"}):
            hpecp = self.cli.CLI()
            hpecp.catalog.list(
                columns=["label_name", "distro_id"], output="text"
            )

        output = self.out.getvalue().strip()
        self.assertEqual(output, "Spark240  bluedata/spark240juphub7xssl")

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list_with_query_and_json_ouput(self, mock_post, mock_get):

        self.maxDiff = None

        hpecp = self.cli.CLI()
        hpecp.catalog.list(
            query="[?state!='installed' && state!='installing'] | [*].[_links.self.href] | []",
            output="json",
        )

        output = self.out.getvalue().strip()

        try:
            json.loads(output)
        except Exception:
            self.fail("Output should be valid json")

        self.assertEqual(output, '["/api/v1/catalog/29"]')

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list_with_query_and_text_output(self, mock_post, mock_get):

        self.maxDiff = None

        with patch.dict("os.environ", {"LOG_LEVEL": "DEBUG"}):
            hpecp = self.cli.CLI()
            hpecp.catalog.list(
                query="[?state!='installed' && state!='installing'] | [*].[_links.self.href] | []",
                output="text",
            )

        output = self.out.getvalue().strip()

        self.assertEqual(output, "/api/v1/catalog/29")


class TestCLIGet(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_get_output_is_valid_yaml(self, mock_post, mock_get):

        self.maxDiff = None

        hpecp = self.cli.CLI()
        hpecp.catalog.get("/api/v1/catalog/100")

        output = self.out.getvalue().strip()
        try:
            yaml.load(output, Loader=yaml.FullLoader)
        except Exception:
            self.fail("Output should be valid yaml")

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_get_yaml_output_is_valid(self, mock_post, mock_get):

        self.maxDiff = None

        hpecp = self.cli.CLI()
        hpecp.catalog.get("/api/v1/catalog/100")

        output = self.out.getvalue().strip()
        try:
            yaml.load(output, Loader=yaml.FullLoader)
        except Exception:
            self.fail("Output should be valid yaml")

        expected_yaml = dedent(
            """\
_links:
    feed:
         - href: https://s3.amazonaws.com/bluedata-catalog/bundles/catalog/external/docker/EPIC-5.0/feeds/feed.json
           name: BlueData EPIC-5.0 catalog feed for docker
    self:
        href: /api/v1/catalog/100
distro_id: bluedata/spark240juphub7xssl
documentation:
    checksum: 52f53f1b2845463b9e370d17fb80bea6
    file: /opt/bluedata/catalog/documentation/bluedata-spark240juphub7xssl-2.8
    mimetype: text/markdown
id: /api/v1/catalog/100
isdebug: false
label:
    description: Spark240 multirole with Jupyter Notebook, Jupyterhub with SSL and gateway node
    name: Spark240
logo:
    checksum: 1471eb59356066ed4a06130566764ea6
    url: http://10.1.0.53/catalog/logos/bluedata-spark240juphub7xssl-2.8
osclass:
     - centos
state: initialized
state_info: ''
timestamp: 0
version: '2.8'"""
        )

        # remove spaces to make testing easier
        self.assertEqual(
            yaml.dump(yaml.load(output, Loader=yaml.FullLoader)),
            yaml.dump(yaml.load(expected_yaml, Loader=yaml.FullLoader)),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_get_json_output(self, mock_post, mock_get):

        self.maxDiff = None

        hpecp = self.cli.CLI()
        hpecp.catalog.get("/api/v1/catalog/100", output="json")

        output = self.out.getvalue().strip()

        # remove spaces to make testing easier
        self.assertEqual(
            json.loads(output),
            {
                "timestamp": 0,
                "logo": {
                    "url": "http://10.1.0.53/catalog/logos/bluedata-spark240juphub7xssl-2.8",
                    "checksum": "1471eb59356066ed4a06130566764ea6",
                },
                "osclass": ["centos"],
                "id": "/api/v1/catalog/100",
                "state_info": "",
                "isdebug": False,
                "documentation": {
                    "mimetype": "text/markdown",
                    "checksum": "52f53f1b2845463b9e370d17fb80bea6",
                    "file": "/opt/bluedata/catalog/documentation/bluedata-spark240juphub7xssl-2.8",
                },
                "distro_id": "bluedata/spark240juphub7xssl",
                "label": {
                    "name": "Spark240",
                    "description": (
                        "Spark240 multirole with Jupyter Notebook, Jupyterhub"
                        " with SSL and gateway node"
                    ),
                },
                "state": "initialized",
                "version": "2.8",
                "_links": {
                    "feed": [
                        {
                            "href": "https://s3.amazonaws.com/bluedata-catalog/bundles/catalog/external/docker/EPIC-5.0/feeds/feed.json",
                            "name": "BlueData EPIC-5.0 catalog feed for docker",
                        }
                    ],
                    "self": {"href": "/api/v1/catalog/100"},
                },
            },
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_get_output_with_invalid_catalog_id(self, mock_post, mock_get):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.catalog.get("/api/v1/catalog/101")

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""  # we don't want error output going to stdout
        expected_stderr = "catalog not found with id: /api/v1/catalog/101"

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(
            stderr.endswith(expected_stderr),
            "expected: `{}` actual: `{}`".format(expected_stderr, stderr),
        )

    def mocked_requests_garbage_data(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/catalog/100":
            return MockResponse(
                json_data={"garbage"},
                status_code=200,
                headers=dict(),
            )
        raise RuntimeError("Unhandle GET request: " + args[0])

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_get_output_with_unknown_exception(self, mock_post, mock_get):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.catalog.get("/api/v1/catalog/102")

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""  # we don't want error output going to stdout
        expected_stderr = (
            "Unknown error. To debug run with env var LOG_LEVEL=DEBUG"
        )

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(stderr.endswith(expected_stderr))


class TestCLIDelete(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_delete(self, mock_post):

        with self.assertRaisesRegexp(
            AttributeError,
            "'CatalogProxy' object has no attribute 'delete'",
        ):
            hpecp = self.cli.CLI()
            hpecp.catalog.delete("/any/id")



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


from .base import BaseTestCase, MockResponse


def mockApiSetup():

    # Mock http (not https) session
    BaseTestCase.registerHttpPostHandler(
        "http://127.0.0.1:8080/api/v1/login",
        MockResponse(
            json_data={},
            status_code=200,
            headers={
                "location": (
                    "/api/v1/session/df1bfacb-ssl-false-xxxx-c8f57d8f3c72"
                )
            },
        ),
    )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


import json

import six
from mock import patch

from .base import BaseTestCase
from .license_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestCLI(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list(self, mock_post, mock_get):

        try:
            hpecp = self.cli.CLI()
            hpecp.license.list()
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = """\
Enabled: true
Licenses:
- Capacity: 240
  DeviceID: 1234 1234
  Evaluation: false
  Expiration: 1609286399000
  ExpirationDisplay: \'2020-12-29T23:59:59Z\'
  Feature: HPE Machine Learning Ops
  Label: The License
  LicenseKey: TEST_LICENSE_KEY
  Start: 1566864000000
  StartDisplay: \'2019-08-27T00:00:00Z\'
  UnlimitedCapacity: false
Messages: []
RevalidateTime: 1609286400000
Summaries:
- AvailableCapacity: 216
  Label: HPE Container Platform
  LatestExpiration: 1609286399000
  LatestExpirationDisplay: \'2020-12-29T23:59:59Z\'
  NextExpiration: 1609286399000
  NextExpirationDisplay: \'2020-12-29T23:59:59Z\'
  RevalidateTime: 1609286400000
  TotalCapacity: 240
  UnlimitedCapacity: false
  UsedCapacity: 24
  Valid: true
  ValidationTime: 1594758782000
- AvailableCapacity: 240
  Label: HPE Machine Learning Ops
  LatestExpiration: 1609286399000
  LatestExpirationDisplay: \'2020-12-29T23:59:59Z\'
  NextExpiration: 1609286399000
  NextExpirationDisplay: \'2020-12-29T23:59:59Z\'
  RevalidateTime: 1609286400000
  TotalCapacity: 240
  UnlimitedCapacity: false
  UsedCapacity: 0
  Valid: true
  ValidationTime: 1594758782000
Valid: true
ValidationTime: 1594758782000
_links:
  self:
    href: /api/v2/hpelicense"""

        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list_license_key_only(self, mock_post, mock_get):

        try:
            hpecp = self.cli.CLI()
            hpecp.license.list(license_key_only=True)
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = "TEST_LICENSE_KEY"
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list_output_json(self, mock_post, mock_get):

        self.maxDiff = None

        try:
            hpecp = self.cli.CLI()
            hpecp.license.list(output="json")
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        stdout = json.dumps(json.loads(stdout), sort_keys=True)
        expected_stdout = json.dumps(
            {
                "_links": {"self": {"href": "/api/v2/hpelicense"}},
                "RevalidateTime": 1609286400000,
                "ValidationTime": 1594758782000,
                "Valid": True,
                "Licenses": [
                    {
                        "Expiration": 1609286399000,
                        "DeviceID": "1234 1234",
                        "Evaluation": False,
                        "Capacity": 240,
                        "UnlimitedCapacity": False,
                        "StartDisplay": "2019-08-27T00:00:00Z",
                        "Start": 1566864000000,
                        "LicenseKey": "TEST_LICENSE_KEY",
                        "ExpirationDisplay": "2020-12-29T23:59:59Z",
                        "Feature": "HPE Machine Learning Ops",
                        "Label": "The License",
                    }
                ],
                "Messages": [],
                "Enabled": True,
                "Summaries": [
                    {
                        "NextExpiration": 1609286399000,
                        "LatestExpirationDisplay": "2020-12-29T23:59:59Z",
                        "TotalCapacity": 240,
                        "Valid": True,
                        "UnlimitedCapacity": False,
                        "UsedCapacity": 24,
                        "RevalidateTime": 1609286400000,
                        "ValidationTime": 1594758782000,
                        "AvailableCapacity": 216,
                        "Label": "HPE Container Platform",
                        "NextExpirationDisplay": "2020-12-29T23:59:59Z",
                        "LatestExpiration": 1609286399000,
                    },
                    {
                        "NextExpiration": 1609286399000,
                        "LatestExpirationDisplay": "2020-12-29T23:59:59Z",
                        "TotalCapacity": 240,
                        "Valid": True,
                        "UnlimitedCapacity": False,
                        "UsedCapacity": 0,
                        "RevalidateTime": 1609286400000,
                        "ValidationTime": 1594758782000,
                        "AvailableCapacity": 240,
                        "Label": "HPE Machine Learning Ops",
                        "NextExpirationDisplay": "2020-12-29T23:59:59Z",
                        "LatestExpiration": 1609286399000,
                    },
                ],
            },
            sort_keys=True,
        )
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_platform_id(self, mock_post, mock_get):

        try:
            hpecp = self.cli.CLI()
            hpecp.license.platform_id()
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = """3c831f6e-f76f-410d-977c-ed13b0c817d1"""
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_register(self, mock_post, mock_get):

        try:
            hpecp = self.cli.CLI()
            hpecp.license.register(server_filename="abc")
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = "/api/v2/hpeclicense/1"
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.delete", side_effect=BaseTestCase.httpDeleteHandlers)
    def test_delete(self, mock_post, mock_delete):

        with patch.dict("os.environ", {"LOG_LEVEL": "DEBUG"}):
            hpecp = self.cli.CLI()
            hpecp.license.delete(license_key="TEST_LICENSE_KEY")

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.delete", side_effect=BaseTestCase.httpDeleteHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_delete_all(self, mock_post, mock_delete, mock_get):

        with patch.dict("os.environ", {"LOG_LEVEL": "DEBUG"}):
            hpecp = self.cli.CLI()
            hpecp.license.delete_all()

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from mock import patch

from hpecp import ContainerPlatformClient

from .base import BaseTestCase
from .config_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestTentants(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_epic_tenant_list(self, mock_post):

        client = ContainerPlatformClient(
            username="admin",
            password="admin123",
            api_host="127.0.0.1",
            api_port=8080,
            use_ssl=True,
        )
        client.create_session()

        client.config.auth(
            {
                "external_identity_server": {
                    "bind_pwd": "5ambaPwd@",
                    "user_attribute": "sAMAccountName",
                    "bind_type": "search_bind",
                    "bind_dn": (
                        "cn=Administrator,CN=Users,DC=samdom"
                        ",DC=example,DC=com"
                    ),
                    "host": "1.1.1.1",
                    "security_protocol": "ldaps",
                    "base_dn": "CN=Users,DC=samdom,DC=example,DC=com",
                    "verify_peer": False,
                    "type": "Active Directory",
                    "port": 636,
                }
            }
        )
